@@ -1,6 +1,8 @@
+# mypy: allow-untyped-defs, allow-incomplete-defs, allow-untyped-calls
+# mypy: no-warn-return-any, allow-any-generics
+
 from __future__ import annotations
 
-from collections import OrderedDict
 from io import StringIO
 import re
 from typing import Any
@@ -17,6 +19,7 @@ from sqlalchemy import schema as sa_schema
 from sqlalchemy import sql
 from sqlalchemy import types as sqltypes
 from sqlalchemy.sql.elements import conv
+from sqlalchemy.sql.elements import quoted_name
 
 from .. import util
 from ..operations import ops
@@ -25,13 +28,12 @@ from ..util import sqla_compat
 if TYPE_CHECKING:
     from typing import Literal
 
+    from sqlalchemy.sql.base import DialectKWArgs
     from sqlalchemy.sql.elements import ColumnElement
-    from sqlalchemy.sql.elements import quoted_name
     from sqlalchemy.sql.elements import TextClause
     from sqlalchemy.sql.schema import CheckConstraint
     from sqlalchemy.sql.schema import Column
     from sqlalchemy.sql.schema import Constraint
-    from sqlalchemy.sql.schema import DefaultClause
     from sqlalchemy.sql.schema import FetchedValue
     from sqlalchemy.sql.schema import ForeignKey
     from sqlalchemy.sql.schema import ForeignKeyConstraint
@@ -54,13 +56,13 @@ MAX_PYTHON_ARGS = 255
 
 
 def _render_gen_name(
-    autogen_context: "AutogenContext",
-    name: Optional[Union["quoted_name", str]],
-) -> Optional[Union["quoted_name", str, "_f_name"]]:
+    autogen_context: AutogenContext,
+    name: sqla_compat._ConstraintName,
+) -> Optional[Union[quoted_name, str, _f_name]]:
     if isinstance(name, conv):
         return _f_name(_alembic_autogenerate_prefix(autogen_context), name)
     else:
-        return name
+        return sqla_compat.constraint_name_or_none(name)
 
 
 def _indent(text: str) -> str:
@@ -70,9 +72,9 @@ def _indent(text: str) -> str:
 
 
 def _render_python_into_templatevars(
-    autogen_context: "AutogenContext",
-    migration_script: "MigrationScript",
-    template_args: Dict[str, Union[str, "Config"]],
+    autogen_context: AutogenContext,
+    migration_script: MigrationScript,
+    template_args: Dict[str, Union[str, Config]],
 ) -> None:
     imports = autogen_context.imports
 
@@ -92,10 +94,9 @@ default_renderers = renderers = util.Dispatcher()
 
 
 def _render_cmd_body(
-    op_container: "ops.OpContainer",
-    autogen_context: "AutogenContext",
+    op_container: ops.OpContainer,
+    autogen_context: AutogenContext,
 ) -> str:
-
     buf = StringIO()
     printer = PythonPrinter(buf)
 
@@ -120,7 +121,7 @@ def _render_cmd_body(
 
 
 def render_op(
-    autogen_context: "AutogenContext", op: "ops.MigrateOperation"
+    autogen_context: AutogenContext, op: ops.MigrateOperation
 ) -> List[str]:
     renderer = renderers.dispatch(op)
     lines = util.to_list(renderer(autogen_context, op))
@@ -128,14 +129,14 @@ def render_op(
 
 
 def render_op_text(
-    autogen_context: "AutogenContext", op: "ops.MigrateOperation"
+    autogen_context: AutogenContext, op: ops.MigrateOperation
 ) -> str:
     return "\n".join(render_op(autogen_context, op))
 
 
 @renderers.dispatch_for(ops.ModifyTableOps)
 def _render_modify_table(
-    autogen_context: "AutogenContext", op: "ModifyTableOps"
+    autogen_context: AutogenContext, op: ModifyTableOps
 ) -> List[str]:
     opts = autogen_context.opts
     render_as_batch = opts.get("render_as_batch", False)
@@ -164,17 +165,24 @@ def _render_modify_table(
 
 @renderers.dispatch_for(ops.CreateTableCommentOp)
 def _render_create_table_comment(
-    autogen_context: "AutogenContext", op: "ops.CreateTableCommentOp"
+    autogen_context: AutogenContext, op: ops.CreateTableCommentOp
 ) -> str:
-
-    templ = (
-        "{prefix}create_table_comment(\n"
-        "{indent}'{tname}',\n"
-        "{indent}{comment},\n"
-        "{indent}existing_comment={existing},\n"
-        "{indent}schema={schema}\n"
-        ")"
-    )
+    if autogen_context._has_batch:
+        templ = (
+            "{prefix}create_table_comment(\n"
+            "{indent}{comment},\n"
+            "{indent}existing_comment={existing}\n"
+            ")"
+        )
+    else:
+        templ = (
+            "{prefix}create_table_comment(\n"
+            "{indent}'{tname}',\n"
+            "{indent}{comment},\n"
+            "{indent}existing_comment={existing},\n"
+            "{indent}schema={schema}\n"
+            ")"
+        )
     return templ.format(
         prefix=_alembic_autogenerate_prefix(autogen_context),
         tname=op.table_name,
@@ -189,16 +197,22 @@ def _render_create_table_comment(
 
 @renderers.dispatch_for(ops.DropTableCommentOp)
 def _render_drop_table_comment(
-    autogen_context: "AutogenContext", op: "ops.DropTableCommentOp"
+    autogen_context: AutogenContext, op: ops.DropTableCommentOp
 ) -> str:
-
-    templ = (
-        "{prefix}drop_table_comment(\n"
-        "{indent}'{tname}',\n"
-        "{indent}existing_comment={existing},\n"
-        "{indent}schema={schema}\n"
-        ")"
-    )
+    if autogen_context._has_batch:
+        templ = (
+            "{prefix}drop_table_comment(\n"
+            "{indent}existing_comment={existing}\n"
+            ")"
+        )
+    else:
+        templ = (
+            "{prefix}drop_table_comment(\n"
+            "{indent}'{tname}',\n"
+            "{indent}existing_comment={existing},\n"
+            "{indent}schema={schema}\n"
+            ")"
+        )
     return templ.format(
         prefix=_alembic_autogenerate_prefix(autogen_context),
         tname=op.table_name,
@@ -211,9 +225,7 @@ def _render_drop_table_comment(
 
 
 @renderers.dispatch_for(ops.CreateTableOp)
-def _add_table(
-    autogen_context: "AutogenContext", op: "ops.CreateTableOp"
-) -> str:
+def _add_table(autogen_context: AutogenContext, op: ops.CreateTableOp) -> str:
     table = op.to_table()
 
     args = [
@@ -251,6 +263,11 @@ def _add_table(
     comment = table.comment
     if comment:
         text += ",\ncomment=%r" % _ident(comment)
+
+    info = table.info
+    if info:
+        text += f",\ninfo={info!r}"
+
     for k in sorted(op.kw):
         text += ",\n%s=%r" % (k.replace(" ", "_"), op.kw[k])
 
@@ -263,9 +280,7 @@ def _add_table(
 
 
 @renderers.dispatch_for(ops.DropTableOp)
-def _drop_table(
-    autogen_context: "AutogenContext", op: "ops.DropTableOp"
-) -> str:
+def _drop_table(autogen_context: AutogenContext, op: ops.DropTableOp) -> str:
     text = "%(prefix)sdrop_table(%(tname)r" % {
         "prefix": _alembic_autogenerate_prefix(autogen_context),
         "tname": _ident(op.table_name),
@@ -276,10 +291,17 @@ def _drop_table(
     return text
 
 
+def _render_dialect_kwargs_items(
+    autogen_context: AutogenContext, item: DialectKWArgs
+) -> list[str]:
+    return [
+        f"{key}={_render_potential_expr(val, autogen_context)}"
+        for key, val in item.dialect_kwargs.items()
+    ]
+
+
 @renderers.dispatch_for(ops.CreateIndexOp)
-def _add_index(
-    autogen_context: "AutogenContext", op: "ops.CreateIndexOp"
-) -> str:
+def _add_index(autogen_context: AutogenContext, op: ops.CreateIndexOp) -> str:
     index = op.to_index()
 
     has_batch = autogen_context._has_batch
@@ -296,6 +318,8 @@ def _add_index(
         )
 
     assert index.table is not None
+
+    opts = _render_dialect_kwargs_items(autogen_context, index)
     text = tmpl % {
         "prefix": _alembic_autogenerate_prefix(autogen_context),
         "name": _render_gen_name(autogen_context, index.name),
@@ -307,26 +331,13 @@ def _add_index(
         "schema": (", schema=%r" % _ident(index.table.schema))
         if index.table.schema
         else "",
-        "kwargs": (
-            ", "
-            + ", ".join(
-                [
-                    "%s=%s"
-                    % (key, _render_potential_expr(val, autogen_context))
-                    for key, val in index.kwargs.items()
-                ]
-            )
-        )
-        if len(index.kwargs)
-        else "",
+        "kwargs": ", " + ", ".join(opts) if opts else "",
     }
     return text
 
 
 @renderers.dispatch_for(ops.DropIndexOp)
-def _drop_index(
-    autogen_context: "AutogenContext", op: "ops.DropIndexOp"
-) -> str:
+def _drop_index(autogen_context: AutogenContext, op: ops.DropIndexOp) -> str:
     index = op.to_index()
 
     has_batch = autogen_context._has_batch
@@ -338,40 +349,28 @@ def _drop_index(
             "%(prefix)sdrop_index(%(name)r, "
             "table_name=%(table_name)r%(schema)s%(kwargs)s)"
         )
-
+    opts = _render_dialect_kwargs_items(autogen_context, index)
     text = tmpl % {
         "prefix": _alembic_autogenerate_prefix(autogen_context),
         "name": _render_gen_name(autogen_context, op.index_name),
         "table_name": _ident(op.table_name),
         "schema": ((", schema=%r" % _ident(op.schema)) if op.schema else ""),
-        "kwargs": (
-            ", "
-            + ", ".join(
-                [
-                    "%s=%s"
-                    % (key, _render_potential_expr(val, autogen_context))
-                    for key, val in index.kwargs.items()
-                ]
-            )
-        )
-        if len(index.kwargs)
-        else "",
+        "kwargs": ", " + ", ".join(opts) if opts else "",
     }
     return text
 
 
 @renderers.dispatch_for(ops.CreateUniqueConstraintOp)
 def _add_unique_constraint(
-    autogen_context: "AutogenContext", op: "ops.CreateUniqueConstraintOp"
+    autogen_context: AutogenContext, op: ops.CreateUniqueConstraintOp
 ) -> List[str]:
     return [_uq_constraint(op.to_constraint(), autogen_context, True)]
 
 
 @renderers.dispatch_for(ops.CreateForeignKeyOp)
 def _add_fk_constraint(
-    autogen_context: "AutogenContext", op: "ops.CreateForeignKeyOp"
+    autogen_context: AutogenContext, op: ops.CreateForeignKeyOp
 ) -> str:
-
     args = [repr(_render_gen_name(autogen_context, op.constraint_name))]
     if not autogen_context._has_batch:
         args.append(repr(_ident(op.source_table)))
@@ -390,6 +389,7 @@ def _add_fk_constraint(
         "initially",
         "deferrable",
         "use_alter",
+        "match",
     ]
     if not autogen_context._has_batch:
         kwargs.insert(0, "source_schema")
@@ -418,32 +418,27 @@ def _add_check_constraint(constraint, autogen_context):
 
 @renderers.dispatch_for(ops.DropConstraintOp)
 def _drop_constraint(
-    autogen_context: "AutogenContext", op: "ops.DropConstraintOp"
+    autogen_context: AutogenContext, op: ops.DropConstraintOp
 ) -> str:
+    prefix = _alembic_autogenerate_prefix(autogen_context)
+    name = _render_gen_name(autogen_context, op.constraint_name)
+    schema = _ident(op.schema) if op.schema else None
+    type_ = _ident(op.constraint_type) if op.constraint_type else None
 
-    if autogen_context._has_batch:
-        template = "%(prefix)sdrop_constraint" "(%(name)r, type_=%(type)r)"
-    else:
-        template = (
-            "%(prefix)sdrop_constraint"
-            "(%(name)r, '%(table_name)s'%(schema)s, type_=%(type)r)"
-        )
+    params_strs = []
+    params_strs.append(repr(name))
+    if not autogen_context._has_batch:
+        params_strs.append(repr(_ident(op.table_name)))
+        if schema is not None:
+            params_strs.append(f"schema={schema!r}")
+    if type_ is not None:
+        params_strs.append(f"type_={type_!r}")
 
-    text = template % {
-        "prefix": _alembic_autogenerate_prefix(autogen_context),
-        "name": _render_gen_name(autogen_context, op.constraint_name),
-        "table_name": _ident(op.table_name),
-        "type": op.constraint_type,
-        "schema": (", schema=%r" % _ident(op.schema)) if op.schema else "",
-    }
-    return text
+    return f"{prefix}drop_constraint({', '.join(params_strs)})"
 
 
 @renderers.dispatch_for(ops.AddColumnOp)
-def _add_column(
-    autogen_context: "AutogenContext", op: "ops.AddColumnOp"
-) -> str:
-
+def _add_column(autogen_context: AutogenContext, op: ops.AddColumnOp) -> str:
     schema, tname, column = op.schema, op.table_name, op.column
     if autogen_context._has_batch:
         template = "%(prefix)sadd_column(%(column)s)"
@@ -462,10 +457,7 @@ def _add_column(
 
 
 @renderers.dispatch_for(ops.DropColumnOp)
-def _drop_column(
-    autogen_context: "AutogenContext", op: "ops.DropColumnOp"
-) -> str:
-
+def _drop_column(autogen_context: AutogenContext, op: ops.DropColumnOp) -> str:
     schema, tname, column_name = op.schema, op.table_name, op.column_name
 
     if autogen_context._has_batch:
@@ -487,9 +479,8 @@ def _drop_column(
 
 @renderers.dispatch_for(ops.AlterColumnOp)
 def _alter_column(
-    autogen_context: "AutogenContext", op: "ops.AlterColumnOp"
+    autogen_context: AutogenContext, op: ops.AlterColumnOp
 ) -> str:
-
     tname = op.table_name
     cname = op.column_name
     server_default = op.modify_server_default
@@ -556,7 +547,7 @@ class _f_name:
         return "%sf(%r)" % (self.prefix, _ident(self.name))
 
 
-def _ident(name: Optional[Union["quoted_name", str]]) -> Optional[str]:
+def _ident(name: Optional[Union[quoted_name, str]]) -> Optional[str]:
     """produce a __repr__() object for a string identifier that may
     use quoted_name() in SQLAlchemy 0.9 and greater.
 
@@ -566,7 +557,7 @@ def _ident(name: Optional[Union["quoted_name", str]]) -> Optional[str]:
     """
     if name is None:
         return name
-    elif isinstance(name, sql.elements.quoted_name):
+    elif isinstance(name, quoted_name):
         return str(name)
     elif isinstance(name, str):
         return name
@@ -574,12 +565,13 @@ def _ident(name: Optional[Union["quoted_name", str]]) -> Optional[str]:
 
 def _render_potential_expr(
     value: Any,
-    autogen_context: "AutogenContext",
+    autogen_context: AutogenContext,
+    *,
     wrap_in_text: bool = True,
     is_server_default: bool = False,
+    is_index: bool = False,
 ) -> str:
     if isinstance(value, sql.ClauseElement):
-
         if wrap_in_text:
             template = "%(prefix)stext(%(sql)r)"
         else:
@@ -588,7 +580,7 @@ def _render_potential_expr(
         return template % {
             "prefix": _sqlalchemy_autogenerate_prefix(autogen_context),
             "sql": autogen_context.migration_context.impl.render_ddl_sql_expr(
-                value, is_server_default=is_server_default
+                value, is_server_default=is_server_default, is_index=is_index
             ),
         }
 
@@ -597,19 +589,19 @@ def _render_potential_expr(
 
 
 def _get_index_rendered_expressions(
-    idx: "Index", autogen_context: "AutogenContext"
+    idx: Index, autogen_context: AutogenContext
 ) -> List[str]:
     return [
         repr(_ident(getattr(exp, "name", None)))
         if isinstance(exp, sa_schema.Column)
-        else _render_potential_expr(exp, autogen_context)
+        else _render_potential_expr(exp, autogen_context, is_index=True)
         for exp in idx.expressions
     ]
 
 
 def _uq_constraint(
-    constraint: "UniqueConstraint",
-    autogen_context: "AutogenContext",
+    constraint: UniqueConstraint,
+    autogen_context: AutogenContext,
     alter: bool,
 ) -> str:
     opts: List[Tuple[str, Any]] = []
@@ -626,6 +618,7 @@ def _uq_constraint(
         opts.append(
             ("name", _render_gen_name(autogen_context, constraint.name))
         )
+    dialect_options = _render_dialect_kwargs_items(autogen_context, constraint)
 
     if alter:
         args = [repr(_render_gen_name(autogen_context, constraint.name))]
@@ -633,6 +626,7 @@ def _uq_constraint(
             args += [repr(_ident(constraint.table.name))]
         args.append(repr([_ident(col.name) for col in constraint.columns]))
         args.extend(["%s=%r" % (k, v) for k, v in opts])
+        args.extend(dialect_options)
         return "%(prefix)screate_unique_constraint(%(args)s)" % {
             "prefix": _alembic_autogenerate_prefix(autogen_context),
             "args": ", ".join(args),
@@ -640,6 +634,7 @@ def _uq_constraint(
     else:
         args = [repr(_ident(col.name)) for col in constraint.columns]
         args.extend(["%s=%r" % (k, v) for k, v in opts])
+        args.extend(dialect_options)
         return "%(prefix)sUniqueConstraint(%(args)s)" % {
             "prefix": _sqlalchemy_autogenerate_prefix(autogen_context),
             "args": ", ".join(args),
@@ -654,11 +649,11 @@ def _user_autogenerate_prefix(autogen_context, target):
         return prefix
 
 
-def _sqlalchemy_autogenerate_prefix(autogen_context: "AutogenContext") -> str:
+def _sqlalchemy_autogenerate_prefix(autogen_context: AutogenContext) -> str:
     return autogen_context.opts["sqlalchemy_module_prefix"] or ""
 
 
-def _alembic_autogenerate_prefix(autogen_context: "AutogenContext") -> str:
+def _alembic_autogenerate_prefix(autogen_context: AutogenContext) -> str:
     if autogen_context._has_batch:
         return "batch_op."
     else:
@@ -666,8 +661,8 @@ def _alembic_autogenerate_prefix(autogen_context: "AutogenContext") -> str:
 
 
 def _user_defined_render(
-    type_: str, object_: Any, autogen_context: "AutogenContext"
-) -> Union[str, "Literal[False]"]:
+    type_: str, object_: Any, autogen_context: AutogenContext
+) -> Union[str, Literal[False]]:
     if "render_item" in autogen_context.opts:
         render = autogen_context.opts["render_item"]
         if render:
@@ -677,7 +672,9 @@ def _user_defined_render(
     return False
 
 
-def _render_column(column: "Column", autogen_context: "AutogenContext") -> str:
+def _render_column(
+    column: Column[Any], autogen_context: AutogenContext
+) -> str:
     rendered = _user_defined_render("column", column, autogen_context)
     if rendered is not False:
         return rendered
@@ -686,7 +683,6 @@ def _render_column(column: "Column", autogen_context: "AutogenContext") -> str:
     opts: List[Tuple[str, Any]] = []
 
     if column.server_default:
-
         rendered = _render_server_default(  # type:ignore[assignment]
             column.server_default, autogen_context
         )
@@ -733,9 +729,7 @@ def _render_column(column: "Column", autogen_context: "AutogenContext") -> str:
     }
 
 
-def _should_render_server_default_positionally(
-    server_default: Union["Computed", "DefaultClause"]
-) -> bool:
+def _should_render_server_default_positionally(server_default: Any) -> bool:
     return sqla_compat._server_default_is_computed(
         server_default
     ) or sqla_compat._server_default_is_identity(server_default)
@@ -743,9 +737,9 @@ def _should_render_server_default_positionally(
 
 def _render_server_default(
     default: Optional[
-        Union["FetchedValue", str, "TextClause", "ColumnElement"]
+        Union[FetchedValue, str, TextClause, ColumnElement[Any]]
     ],
-    autogen_context: "AutogenContext",
+    autogen_context: AutogenContext,
     repr_: bool = True,
 ) -> Optional[str]:
     rendered = _user_defined_render("server_default", default, autogen_context)
@@ -771,7 +765,7 @@ def _render_server_default(
 
 
 def _render_computed(
-    computed: "Computed", autogen_context: "AutogenContext"
+    computed: Computed, autogen_context: AutogenContext
 ) -> str:
     text = _render_potential_expr(
         computed.sqltext, autogen_context, wrap_in_text=False
@@ -788,13 +782,11 @@ def _render_computed(
 
 
 def _render_identity(
-    identity: "Identity", autogen_context: "AutogenContext"
+    identity: Identity, autogen_context: AutogenContext
 ) -> str:
-    # always=None means something different than always=False
-    kwargs = OrderedDict(always=identity.always)
-    if identity.on_null is not None:
-        kwargs["on_null"] = identity.on_null
-    kwargs.update(_get_identity_options(identity))
+    kwargs = sqla_compat._get_identity_options_dict(
+        identity, dialect_kwargs=True
+    )
 
     return "%(prefix)sIdentity(%(kwargs)s)" % {
         "prefix": _sqlalchemy_autogenerate_prefix(autogen_context),
@@ -802,18 +794,9 @@ def _render_identity(
     }
 
 
-def _get_identity_options(identity_options: "Identity") -> OrderedDict:
-    kwargs = OrderedDict()
-    for attr in sqla_compat._identity_options_attrs:
-        value = getattr(identity_options, attr, None)
-        if value is not None:
-            kwargs[attr] = value
-    return kwargs
-
-
 def _repr_type(
-    type_: "TypeEngine",
-    autogen_context: "AutogenContext",
+    type_: TypeEngine,
+    autogen_context: AutogenContext,
     _skip_variants: bool = False,
 ) -> str:
     rendered = _user_defined_render("type", type_, autogen_context)
@@ -841,9 +824,9 @@ def _repr_type(
             return "%s.%r" % (dname, type_)
     elif impl_rt:
         return impl_rt
+    elif not _skip_variants and sqla_compat._type_has_variants(type_):
+        return _render_Variant_type(type_, autogen_context)
     elif mod.startswith("sqlalchemy."):
-        if not _skip_variants and sqla_compat._type_has_variants(type_):
-            return _render_Variant_type(type_, autogen_context)
         if "_render_%s_type" % type_.__visit_name__ in globals():
             fn = globals()["_render_%s_type" % type_.__visit_name__]
             return fn(type_, autogen_context)
@@ -855,9 +838,7 @@ def _repr_type(
         return "%s%r" % (prefix, type_)
 
 
-def _render_ARRAY_type(
-    type_: "ARRAY", autogen_context: "AutogenContext"
-) -> str:
+def _render_ARRAY_type(type_: ARRAY, autogen_context: AutogenContext) -> str:
     return cast(
         str,
         _render_type_w_subtype(
@@ -867,11 +848,11 @@ def _render_ARRAY_type(
 
 
 def _render_Variant_type(
-    type_: "TypeEngine", autogen_context: "AutogenContext"
+    type_: TypeEngine, autogen_context: AutogenContext
 ) -> str:
     base_type, variant_mapping = sqla_compat._get_variant_mapping(type_)
     base = _repr_type(base_type, autogen_context, _skip_variants=True)
-    assert base is not None and base is not False
+    assert base is not None and base is not False  # type: ignore[comparison-overlap]  # noqa:E501
     for dialect in sorted(variant_mapping):
         typ = variant_mapping[dialect]
         base += ".with_variant(%s, %r)" % (
@@ -882,12 +863,12 @@ def _render_Variant_type(
 
 
 def _render_type_w_subtype(
-    type_: "TypeEngine",
-    autogen_context: "AutogenContext",
+    type_: TypeEngine,
+    autogen_context: AutogenContext,
     attrname: str,
     regexp: str,
     prefix: Optional[str] = None,
-) -> Union[Optional[str], "Literal[False]"]:
+) -> Union[Optional[str], Literal[False]]:
     outer_repr = repr(type_)
     inner_type = getattr(type_, attrname, None)
     if inner_type is None:
@@ -919,9 +900,9 @@ _constraint_renderers = util.Dispatcher()
 
 
 def _render_constraint(
-    constraint: "Constraint",
-    autogen_context: "AutogenContext",
-    namespace_metadata: Optional["MetaData"],
+    constraint: Constraint,
+    autogen_context: AutogenContext,
+    namespace_metadata: Optional[MetaData],
 ) -> Optional[str]:
     try:
         renderer = _constraint_renderers.dispatch(constraint)
@@ -934,9 +915,9 @@ def _render_constraint(
 
 @_constraint_renderers.dispatch_for(sa_schema.PrimaryKeyConstraint)
 def _render_primary_key(
-    constraint: "PrimaryKeyConstraint",
-    autogen_context: "AutogenContext",
-    namespace_metadata: Optional["MetaData"],
+    constraint: PrimaryKeyConstraint,
+    autogen_context: AutogenContext,
+    namespace_metadata: Optional[MetaData],
 ) -> Optional[str]:
     rendered = _user_defined_render("primary_key", constraint, autogen_context)
     if rendered is not False:
@@ -960,15 +941,15 @@ def _render_primary_key(
 
 
 def _fk_colspec(
-    fk: "ForeignKey",
+    fk: ForeignKey,
     metadata_schema: Optional[str],
-    namespace_metadata: "MetaData",
+    namespace_metadata: MetaData,
 ) -> str:
     """Implement a 'safe' version of ForeignKey._get_colspec() that
     won't fail if the remote table can't be resolved.
 
     """
-    colspec = fk._get_colspec()  # type:ignore[attr-defined]
+    colspec = fk._get_colspec()
     tokens = colspec.split(".")
     tname, colname = tokens[-2:]
 
@@ -989,7 +970,7 @@ def _fk_colspec(
         if table_fullname in namespace_metadata.tables:
             col = namespace_metadata.tables[table_fullname].c.get(colname)
             if col is not None:
-                colname = _ident(col.name)
+                colname = _ident(col.name)  # type: ignore[assignment]
 
     colspec = "%s.%s" % (table_fullname, colname)
 
@@ -997,9 +978,8 @@ def _fk_colspec(
 
 
 def _populate_render_fk_opts(
-    constraint: "ForeignKeyConstraint", opts: List[Tuple[str, str]]
+    constraint: ForeignKeyConstraint, opts: List[Tuple[str, str]]
 ) -> None:
-
     if constraint.onupdate:
         opts.append(("onupdate", repr(constraint.onupdate)))
     if constraint.ondelete:
@@ -1010,13 +990,15 @@ def _populate_render_fk_opts(
         opts.append(("deferrable", repr(constraint.deferrable)))
     if constraint.use_alter:
         opts.append(("use_alter", repr(constraint.use_alter)))
+    if constraint.match:
+        opts.append(("match", repr(constraint.match)))
 
 
 @_constraint_renderers.dispatch_for(sa_schema.ForeignKeyConstraint)
 def _render_foreign_key(
-    constraint: "ForeignKeyConstraint",
-    autogen_context: "AutogenContext",
-    namespace_metadata: "MetaData",
+    constraint: ForeignKeyConstraint,
+    autogen_context: AutogenContext,
+    namespace_metadata: MetaData,
 ) -> Optional[str]:
     rendered = _user_defined_render("foreign_key", constraint, autogen_context)
     if rendered is not False:
@@ -1037,8 +1019,7 @@ def _render_foreign_key(
         % {
             "prefix": _sqlalchemy_autogenerate_prefix(autogen_context),
             "cols": ", ".join(
-                "%r" % _ident(cast("Column", f.parent).name)
-                for f in constraint.elements
+                repr(_ident(f.parent.name)) for f in constraint.elements
             ),
             "refcols": ", ".join(
                 repr(_fk_colspec(f, apply_metadata_schema, namespace_metadata))
@@ -1053,9 +1034,9 @@ def _render_foreign_key(
 
 @_constraint_renderers.dispatch_for(sa_schema.UniqueConstraint)
 def _render_unique_constraint(
-    constraint: "UniqueConstraint",
-    autogen_context: "AutogenContext",
-    namespace_metadata: Optional["MetaData"],
+    constraint: UniqueConstraint,
+    autogen_context: AutogenContext,
+    namespace_metadata: Optional[MetaData],
 ) -> str:
     rendered = _user_defined_render("unique", constraint, autogen_context)
     if rendered is not False:
@@ -1066,9 +1047,9 @@ def _render_unique_constraint(
 
 @_constraint_renderers.dispatch_for(sa_schema.CheckConstraint)
 def _render_check_constraint(
-    constraint: "CheckConstraint",
-    autogen_context: "AutogenContext",
-    namespace_metadata: Optional["MetaData"],
+    constraint: CheckConstraint,
+    autogen_context: AutogenContext,
+    namespace_metadata: Optional[MetaData],
 ) -> Optional[str]:
     rendered = _user_defined_render("check", constraint, autogen_context)
     if rendered is not False:
@@ -1079,12 +1060,10 @@ def _render_check_constraint(
     # ideally SQLAlchemy would give us more of a first class
     # way to detect this.
     if (
-        constraint._create_rule  # type:ignore[attr-defined]
-        and hasattr(
-            constraint._create_rule, "target"  # type:ignore[attr-defined]
-        )
+        constraint._create_rule
+        and hasattr(constraint._create_rule, "target")
         and isinstance(
-            constraint._create_rule.target,  # type:ignore[attr-defined]
+            constraint._create_rule.target,
             sqltypes.TypeEngine,
         )
     ):
@@ -1106,9 +1085,7 @@ def _render_check_constraint(
 
 
 @renderers.dispatch_for(ops.ExecuteSQLOp)
-def _execute_sql(
-    autogen_context: "AutogenContext", op: "ops.ExecuteSQLOp"
-) -> str:
+def _execute_sql(autogen_context: AutogenContext, op: ops.ExecuteSQLOp) -> str:
     if not isinstance(op.sqltext, str):
         raise NotImplementedError(
             "Autogenerate rendering of SQL Expression language constructs "
