@@ -1,7 +1,8 @@
+# mypy: allow-untyped-defs, allow-untyped-calls
+
 from __future__ import annotations
 
 import os
-from typing import Callable
 from typing import List
 from typing import Optional
 from typing import TYPE_CHECKING
@@ -15,9 +16,11 @@ from .script import ScriptDirectory
 if TYPE_CHECKING:
     from alembic.config import Config
     from alembic.script.base import Script
+    from alembic.script.revision import _RevIdType
+    from .runtime.environment import ProcessRevisionDirectiveFn
 
 
-def list_templates(config):
+def list_templates(config: Config) -> None:
     """List available templates.
 
     :param config: a :class:`.Config` object.
@@ -37,7 +40,7 @@ def list_templates(config):
 
 
 def init(
-    config: "Config",
+    config: Config,
     directory: str,
     template: str = "generic",
     package: bool = False,
@@ -54,9 +57,6 @@ def init(
     :param package: when True, write ``__init__.py`` files into the
      environment location as well as the versions/ location.
 
-     .. versionadded:: 1.2
-
-
     """
 
     if os.access(directory, os.F_OK) and os.listdir(directory):
@@ -69,28 +69,32 @@ def init(
         raise util.CommandError("No such template %r" % template)
 
     if not os.access(directory, os.F_OK):
-        util.status(
-            "Creating directory %s" % os.path.abspath(directory),
-            os.makedirs,
-            directory,
-        )
+        with util.status(
+            f"Creating directory {os.path.abspath(directory)!r}",
+            **config.messaging_opts,
+        ):
+            os.makedirs(directory)
 
     versions = os.path.join(directory, "versions")
-    util.status(
-        "Creating directory %s" % os.path.abspath(versions),
-        os.makedirs,
-        versions,
-    )
+    with util.status(
+        f"Creating directory {os.path.abspath(versions)!r}",
+        **config.messaging_opts,
+    ):
+        os.makedirs(versions)
 
     script = ScriptDirectory(directory)
 
+    config_file: str | None = None
     for file_ in os.listdir(template_dir):
         file_path = os.path.join(template_dir, file_)
         if file_ == "alembic.ini.mako":
             assert config.config_file_name is not None
             config_file = os.path.abspath(config.config_file_name)
             if os.access(config_file, os.F_OK):
-                util.msg("File %s already exists, skipping" % config_file)
+                util.msg(
+                    f"File {config_file!r} already exists, skipping",
+                    **config.messaging_opts,
+                )
             else:
                 script._generate_template(
                     file_path, config_file, script_location=directory
@@ -104,28 +108,31 @@ def init(
             os.path.join(os.path.abspath(directory), "__init__.py"),
             os.path.join(os.path.abspath(versions), "__init__.py"),
         ]:
-            file_ = util.status("Adding %s" % path, open, path, "w")
-            file_.close()  # type:ignore[attr-defined]
+            with util.status(f"Adding {path!r}", **config.messaging_opts):
+                with open(path, "w"):
+                    pass
 
+    assert config_file is not None
     util.msg(
         "Please edit configuration/connection/logging "
-        "settings in %r before proceeding." % config_file
+        f"settings in {config_file!r} before proceeding.",
+        **config.messaging_opts,
     )
 
 
 def revision(
-    config: "Config",
+    config: Config,
     message: Optional[str] = None,
     autogenerate: bool = False,
     sql: bool = False,
     head: str = "head",
     splice: bool = False,
-    branch_label: Optional[str] = None,
+    branch_label: Optional[_RevIdType] = None,
     version_path: Optional[str] = None,
     rev_id: Optional[str] = None,
     depends_on: Optional[str] = None,
-    process_revision_directives: Callable = None,
-) -> Union[Optional["Script"], List[Optional["Script"]]]:
+    process_revision_directives: Optional[ProcessRevisionDirectiveFn] = None,
+) -> Union[Optional[Script], List[Optional[Script]]]:
     """Create a new revision file.
 
     :param config: a :class:`.Config` object.
@@ -240,13 +247,70 @@ def revision(
         return scripts
 
 
+def check(config: "Config") -> None:
+    """Check if revision command with autogenerate has pending upgrade ops.
+
+    :param config: a :class:`.Config` object.
+
+    .. versionadded:: 1.9.0
+
+    """
+
+    script_directory = ScriptDirectory.from_config(config)
+
+    command_args = dict(
+        message=None,
+        autogenerate=True,
+        sql=False,
+        head="head",
+        splice=False,
+        branch_label=None,
+        version_path=None,
+        rev_id=None,
+        depends_on=None,
+    )
+    revision_context = autogen.RevisionContext(
+        config,
+        script_directory,
+        command_args,
+    )
+
+    def retrieve_migrations(rev, context):
+        revision_context.run_autogenerate(rev, context)
+        return []
+
+    with EnvironmentContext(
+        config,
+        script_directory,
+        fn=retrieve_migrations,
+        as_sql=False,
+        template_args=revision_context.template_args,
+        revision_context=revision_context,
+    ):
+        script_directory.run_env()
+
+    # the revision_context now has MigrationScript structure(s) present.
+
+    migration_script = revision_context.generated_revisions[-1]
+    diffs = []
+    for upgrade_ops in migration_script.upgrade_ops_list:
+        diffs.extend(upgrade_ops.as_diffs())
+
+    if diffs:
+        raise util.AutogenerateDiffsDetected(
+            f"New upgrade operations detected: {diffs}"
+        )
+    else:
+        config.print_stdout("No new upgrade operations detected.")
+
+
 def merge(
-    config: "Config",
-    revisions: str,
-    message: str = None,
-    branch_label: str = None,
-    rev_id: str = None,
-) -> Optional["Script"]:
+    config: Config,
+    revisions: _RevIdType,
+    message: Optional[str] = None,
+    branch_label: Optional[_RevIdType] = None,
+    rev_id: Optional[str] = None,
+) -> Optional[Script]:
     """Merge two revisions together.  Creates a new migration file.
 
     :param config: a :class:`.Config` instance
@@ -266,9 +330,26 @@ def merge(
 
     script = ScriptDirectory.from_config(config)
     template_args = {
-        "config": "config"  # Let templates use config for
+        "config": config  # Let templates use config for
         # e.g. multiple databases
     }
+
+    environment = util.asbool(config.get_main_option("revision_environment"))
+
+    if environment:
+
+        def nothing(rev, context):
+            return []
+
+        with EnvironmentContext(
+            config,
+            script,
+            fn=nothing,
+            as_sql=False,
+            template_args=template_args,
+        ):
+            script.run_env()
+
     return script.generate_revision(
         rev_id or util.rev_id(),
         message,
@@ -280,7 +361,7 @@ def merge(
 
 
 def upgrade(
-    config: "Config",
+    config: Config,
     revision: str,
     sql: bool = False,
     tag: Optional[str] = None,
@@ -323,7 +404,7 @@ def upgrade(
 
 
 def downgrade(
-    config: "Config",
+    config: Config,
     revision: str,
     sql: bool = False,
     tag: Optional[str] = None,
@@ -394,7 +475,7 @@ def show(config, rev):
 
 
 def history(
-    config: "Config",
+    config: Config,
     rev_range: Optional[str] = None,
     verbose: bool = False,
     indicate_current: bool = False,
@@ -431,7 +512,6 @@ def history(
         for sc in script.walk_revisions(
             base=base or "base", head=head or "heads"
         ):
-
             if indicate_current:
                 sc._db_current_indicator = sc.revision in currents
 
@@ -517,7 +597,7 @@ def branches(config, verbose=False):
             )
 
 
-def current(config: "Config", verbose: bool = False) -> None:
+def current(config: Config, verbose: bool = False) -> None:
     """Display the current revision for a database.
 
     :param config: a :class:`.Config` instance.
@@ -546,8 +626,8 @@ def current(config: "Config", verbose: bool = False) -> None:
 
 
 def stamp(
-    config: "Config",
-    revision: str,
+    config: Config,
+    revision: _RevIdType,
     sql: bool = False,
     tag: Optional[str] = None,
     purge: bool = False,
@@ -563,9 +643,6 @@ def stamp(
      .. note:: this parameter is called "revisions" in the command line
         interface.
 
-     .. versionchanged:: 1.2  The revision may be a single revision or
-        list of revisions when stamping multiple branch heads.
-
     :param sql: use ``--sql`` mode
 
     :param tag: an arbitrary "tag" that can be intercepted by custom
@@ -574,8 +651,6 @@ def stamp(
 
     :param purge: delete all entries in the version table before stamping.
 
-     .. versionadded:: 1.2
-
     """
 
     script = ScriptDirectory.from_config(config)
@@ -615,7 +690,7 @@ def stamp(
         script.run_env()
 
 
-def edit(config: "Config", rev: str) -> None:
+def edit(config: Config, rev: str) -> None:
     """Edit revision script(s) using $EDITOR.
 
     :param config: a :class:`.Config` instance.
@@ -648,7 +723,7 @@ def edit(config: "Config", rev: str) -> None:
             util.open_in_editor(sc.path)
 
 
-def ensure_version(config: "Config", sql: bool = False) -> None:
+def ensure_version(config: Config, sql: bool = False) -> None:
     """Create the alembic version table if it doesn't exist already .
 
     :param config: a :class:`.Config` instance.
