@@ -1,9 +1,17 @@
-from collections import namedtuple
+# mypy: allow-untyped-defs, allow-incomplete-defs, allow-untyped-calls
+# mypy: no-warn-return-any, allow-any-generics
+
+from __future__ import annotations
+
+import logging
 import re
 from typing import Any
 from typing import Callable
 from typing import Dict
+from typing import Iterable
 from typing import List
+from typing import Mapping
+from typing import NamedTuple
 from typing import Optional
 from typing import Sequence
 from typing import Set
@@ -16,24 +24,25 @@ from sqlalchemy import cast
 from sqlalchemy import schema
 from sqlalchemy import text
 
+from . import _autogen
 from . import base
+from ._autogen import _constraint_sig as _constraint_sig
+from ._autogen import ComparisonResult as ComparisonResult
 from .. import util
 from ..util import sqla_compat
 
 if TYPE_CHECKING:
-    from io import StringIO
     from typing import Literal
+    from typing import TextIO
 
     from sqlalchemy.engine import Connection
     from sqlalchemy.engine import Dialect
     from sqlalchemy.engine.cursor import CursorResult
-    from sqlalchemy.engine.cursor import LegacyCursorResult
     from sqlalchemy.engine.reflection import Inspector
-    from sqlalchemy.sql.dml import Update
-    from sqlalchemy.sql.elements import ClauseElement
+    from sqlalchemy.sql import ClauseElement
+    from sqlalchemy.sql import Executable
     from sqlalchemy.sql.elements import ColumnElement
     from sqlalchemy.sql.elements import quoted_name
-    from sqlalchemy.sql.elements import TextClause
     from sqlalchemy.sql.schema import Column
     from sqlalchemy.sql.schema import Constraint
     from sqlalchemy.sql.schema import ForeignKeyConstraint
@@ -48,23 +57,23 @@ if TYPE_CHECKING:
     from ..operations.batch import ApplyBatchImpl
     from ..operations.batch import BatchOperationsImpl
 
+log = logging.getLogger(__name__)
+
 
 class ImplMeta(type):
     def __init__(
         cls,
         classname: str,
-        bases: Tuple[Type["DefaultImpl"]],
+        bases: Tuple[Type[DefaultImpl]],
         dict_: Dict[str, Any],
     ):
         newtype = type.__init__(cls, classname, bases, dict_)
         if "__dialect__" in dict_:
-            _impls[dict_["__dialect__"]] = cls
+            _impls[dict_["__dialect__"]] = cls  # type: ignore[assignment]
         return newtype
 
 
-_impls: dict = {}
-
-Params = namedtuple("Params", ["token0", "tokens", "args", "kwargs"])
+_impls: Dict[str, Type[DefaultImpl]] = {}
 
 
 class DefaultImpl(metaclass=ImplMeta):
@@ -87,16 +96,19 @@ class DefaultImpl(metaclass=ImplMeta):
     command_terminator = ";"
     type_synonyms: Tuple[Set[str], ...] = ({"NUMERIC", "DECIMAL"},)
     type_arg_extract: Sequence[str] = ()
-    # on_null is known to be supported only by oracle
-    identity_attrs_ignore: Tuple[str, ...] = ("on_null",)
+    # These attributes are deprecated in SQLAlchemy via #10247. They need to
+    # be ignored to support older version that did not use dialect kwargs.
+    # They only apply to Oracle and are replaced by oracle_order,
+    # oracle_on_null
+    identity_attrs_ignore: Tuple[str, ...] = ("order", "on_null")
 
     def __init__(
         self,
-        dialect: "Dialect",
-        connection: Optional["Connection"],
+        dialect: Dialect,
+        connection: Optional[Connection],
         as_sql: bool,
         transactional_ddl: Optional[bool],
-        output_buffer: Optional["StringIO"],
+        output_buffer: Optional[TextIO],
         context_opts: Dict[str, Any],
     ) -> None:
         self.dialect = dialect
@@ -117,7 +129,7 @@ class DefaultImpl(metaclass=ImplMeta):
                 )
 
     @classmethod
-    def get_by_dialect(cls, dialect: "Dialect") -> Any:
+    def get_by_dialect(cls, dialect: Dialect) -> Type[DefaultImpl]:
         return _impls[dialect.name]
 
     def static_output(self, text: str) -> None:
@@ -126,7 +138,7 @@ class DefaultImpl(metaclass=ImplMeta):
         self.output_buffer.flush()
 
     def requires_recreate_in_batch(
-        self, batch_op: "BatchOperationsImpl"
+        self, batch_op: BatchOperationsImpl
     ) -> bool:
         """Return True if the given :class:`.BatchOperationsImpl`
         would need the table to be recreated and copied in order to
@@ -139,7 +151,7 @@ class DefaultImpl(metaclass=ImplMeta):
         return False
 
     def prep_table_for_batch(
-        self, batch_impl: "ApplyBatchImpl", table: "Table"
+        self, batch_impl: ApplyBatchImpl, table: Table
     ) -> None:
         """perform any operations needed on a table before a new
         one is created to replace it in batch mode.
@@ -150,16 +162,16 @@ class DefaultImpl(metaclass=ImplMeta):
         """
 
     @property
-    def bind(self) -> Optional["Connection"]:
+    def bind(self) -> Optional[Connection]:
         return self.connection
 
     def _exec(
         self,
-        construct: Union["ClauseElement", str],
-        execution_options: None = None,
+        construct: Union[Executable, str],
+        execution_options: Optional[dict[str, Any]] = None,
         multiparams: Sequence[dict] = (),
-        params: Dict[str, int] = util.immutabledict(),
-    ) -> Optional[Union["LegacyCursorResult", "CursorResult"]]:
+        params: Dict[str, Any] = util.immutabledict(),
+    ) -> Optional[CursorResult]:
         if isinstance(construct, str):
             construct = text(construct)
         if self.as_sql:
@@ -167,6 +179,7 @@ class DefaultImpl(metaclass=ImplMeta):
                 # TODO: coverage
                 raise Exception("Execution arguments not allowed with as_sql")
 
+            compile_kw: dict[str, Any]
             if self.literal_binds and not isinstance(
                 construct, schema.DDLElement
             ):
@@ -174,10 +187,11 @@ class DefaultImpl(metaclass=ImplMeta):
             else:
                 compile_kw = {}
 
+            if TYPE_CHECKING:
+                assert isinstance(construct, ClauseElement)
+            compiled = construct.compile(dialect=self.dialect, **compile_kw)
             self.static_output(
-                str(construct.compile(dialect=self.dialect, **compile_kw))
-                .replace("\t", "    ")
-                .strip()
+                str(compiled).replace("\t", "    ").strip()
                 + self.command_terminator
             )
             return None
@@ -194,8 +208,8 @@ class DefaultImpl(metaclass=ImplMeta):
 
     def execute(
         self,
-        sql: Union["Update", "TextClause", str],
-        execution_options: None = None,
+        sql: Union[Executable, str],
+        execution_options: Optional[dict[str, Any]] = None,
     ) -> None:
         self._exec(sql, execution_options)
 
@@ -204,18 +218,18 @@ class DefaultImpl(metaclass=ImplMeta):
         table_name: str,
         column_name: str,
         nullable: Optional[bool] = None,
-        server_default: Union["_ServerDefault", "Literal[False]"] = False,
+        server_default: Union[_ServerDefault, Literal[False]] = False,
         name: Optional[str] = None,
-        type_: Optional["TypeEngine"] = None,
+        type_: Optional[TypeEngine] = None,
         schema: Optional[str] = None,
         autoincrement: Optional[bool] = None,
-        comment: Optional[Union[str, "Literal[False]"]] = False,
+        comment: Optional[Union[str, Literal[False]]] = False,
         existing_comment: Optional[str] = None,
-        existing_type: Optional["TypeEngine"] = None,
-        existing_server_default: Optional["_ServerDefault"] = None,
+        existing_type: Optional[TypeEngine] = None,
+        existing_server_default: Optional[_ServerDefault] = None,
         existing_nullable: Optional[bool] = None,
         existing_autoincrement: Optional[bool] = None,
-        **kw: Any
+        **kw: Any,
     ) -> None:
         if autoincrement is not None or existing_autoincrement is not None:
             util.warn(
@@ -266,7 +280,7 @@ class DefaultImpl(metaclass=ImplMeta):
                     existing_server_default=existing_server_default,
                     existing_nullable=existing_nullable,
                     existing_comment=existing_comment,
-                    **kw
+                    **kw,
                 )
             )
         if type_ is not None:
@@ -314,17 +328,17 @@ class DefaultImpl(metaclass=ImplMeta):
     def add_column(
         self,
         table_name: str,
-        column: "Column",
-        schema: Optional[Union[str, "quoted_name"]] = None,
+        column: Column[Any],
+        schema: Optional[Union[str, quoted_name]] = None,
     ) -> None:
         self._exec(base.AddColumn(table_name, column, schema=schema))
 
     def drop_column(
         self,
         table_name: str,
-        column: "Column",
+        column: Column[Any],
         schema: Optional[str] = None,
-        **kw
+        **kw,
     ) -> None:
         self._exec(base.DropColumn(table_name, column, schema=schema))
 
@@ -332,20 +346,20 @@ class DefaultImpl(metaclass=ImplMeta):
         if const._create_rule is None or const._create_rule(self):
             self._exec(schema.AddConstraint(const))
 
-    def drop_constraint(self, const: "Constraint") -> None:
+    def drop_constraint(self, const: Constraint) -> None:
         self._exec(schema.DropConstraint(const))
 
     def rename_table(
         self,
         old_table_name: str,
-        new_table_name: Union[str, "quoted_name"],
-        schema: Optional[Union[str, "quoted_name"]] = None,
+        new_table_name: Union[str, quoted_name],
+        schema: Optional[Union[str, quoted_name]] = None,
     ) -> None:
         self._exec(
             base.RenameTable(old_table_name, new_table_name, schema=schema)
         )
 
-    def create_table(self, table: "Table") -> None:
+    def create_table(self, table: Table) -> None:
         table.dispatch.before_create(
             table, self.connection, checkfirst=False, _ddl_runner=self
         )
@@ -368,27 +382,33 @@ class DefaultImpl(metaclass=ImplMeta):
             if comment and with_comment:
                 self.create_column_comment(column)
 
-    def drop_table(self, table: "Table") -> None:
+    def drop_table(self, table: Table) -> None:
+        table.dispatch.before_drop(
+            table, self.connection, checkfirst=False, _ddl_runner=self
+        )
         self._exec(schema.DropTable(table))
+        table.dispatch.after_drop(
+            table, self.connection, checkfirst=False, _ddl_runner=self
+        )
 
-    def create_index(self, index: "Index") -> None:
-        self._exec(schema.CreateIndex(index))
+    def create_index(self, index: Index, **kw: Any) -> None:
+        self._exec(schema.CreateIndex(index, **kw))
 
-    def create_table_comment(self, table: "Table") -> None:
+    def create_table_comment(self, table: Table) -> None:
         self._exec(schema.SetTableComment(table))
 
-    def drop_table_comment(self, table: "Table") -> None:
+    def drop_table_comment(self, table: Table) -> None:
         self._exec(schema.DropTableComment(table))
 
-    def create_column_comment(self, column: "ColumnElement") -> None:
+    def create_column_comment(self, column: ColumnElement[Any]) -> None:
         self._exec(schema.SetColumnComment(column))
 
-    def drop_index(self, index: "Index") -> None:
-        self._exec(schema.DropIndex(index))
+    def drop_index(self, index: Index, **kw: Any) -> None:
+        self._exec(schema.DropIndex(index, **kw))
 
     def bulk_insert(
         self,
-        table: Union["TableClause", "Table"],
+        table: Union[TableClause, Table],
         rows: List[dict],
         multiinsert: bool = True,
     ) -> None:
@@ -400,25 +420,19 @@ class DefaultImpl(metaclass=ImplMeta):
             for row in rows:
                 self._exec(
                     sqla_compat._insert_inline(table).values(
-                        **dict(
-                            (
-                                k,
-                                sqla_compat._literal_bindparam(
-                                    k, v, type_=table.c[k].type
-                                )
-                                if not isinstance(
-                                    v, sqla_compat._literal_bindparam
-                                )
-                                else v,
+                        **{
+                            k: sqla_compat._literal_bindparam(
+                                k, v, type_=table.c[k].type
                             )
+                            if not isinstance(
+                                v, sqla_compat._literal_bindparam
+                            )
+                            else v
                             for k, v in row.items()
-                        )
+                        }
                     )
                 )
         else:
-            # work around http://www.sqlalchemy.org/trac/ticket/2461
-            if not hasattr(table, "_autoincrement_column"):
-                table._autoincrement_column = None
             if rows:
                 if multiinsert:
                     self._exec(
@@ -430,7 +444,8 @@ class DefaultImpl(metaclass=ImplMeta):
                             sqla_compat._insert_inline(table).values(**row)
                         )
 
-    def _tokenize_column_type(self, column: "Column") -> Params:
+    def _tokenize_column_type(self, column: Column) -> Params:
+        definition: str
         definition = self.dialect.type_compiler.process(column.type).lower()
 
         # tokenize the SQLAlchemy-generated version of a type, so that
@@ -445,9 +460,9 @@ class DefaultImpl(metaclass=ImplMeta):
         # varchar character set utf8
         #
 
-        tokens = re.findall(r"[\w\-_]+|\(.+?\)", definition)
+        tokens: List[str] = re.findall(r"[\w\-_]+|\(.+?\)", definition)
 
-        term_tokens = []
+        term_tokens: List[str] = []
         paren_term = None
 
         for token in tokens:
@@ -459,6 +474,7 @@ class DefaultImpl(metaclass=ImplMeta):
         params = Params(term_tokens[0], term_tokens[1:], [], {})
 
         if paren_term:
+            term: str
             for term in re.findall("[^(),]+", paren_term):
                 if "=" in term:
                     key, val = term.split("=")
@@ -469,7 +485,7 @@ class DefaultImpl(metaclass=ImplMeta):
         return params
 
     def _column_types_match(
-        self, inspector_params: "Params", metadata_params: "Params"
+        self, inspector_params: Params, metadata_params: Params
     ) -> bool:
         if inspector_params.token0 == metadata_params.token0:
             return True
@@ -491,7 +507,7 @@ class DefaultImpl(metaclass=ImplMeta):
         return False
 
     def _column_args_match(
-        self, inspected_params: "Params", meta_params: "Params"
+        self, inspected_params: Params, meta_params: Params
     ) -> bool:
         """We want to compare column parameters. However, we only want
         to compare parameters that are set. If they both have `collation`,
@@ -524,7 +540,7 @@ class DefaultImpl(metaclass=ImplMeta):
         return True
 
     def compare_type(
-        self, inspector_column: "Column", metadata_column: "Column"
+        self, inspector_column: Column[Any], metadata_column: Column
     ) -> bool:
         """Returns True if there ARE differences between the types of the two
         columns. Takes impl.type_synonyms into account between retrospected
@@ -550,10 +566,10 @@ class DefaultImpl(metaclass=ImplMeta):
 
     def correct_for_autogen_constraints(
         self,
-        conn_uniques: Set["UniqueConstraint"],
-        conn_indexes: Set["Index"],
-        metadata_unique_constraints: Set["UniqueConstraint"],
-        metadata_indexes: Set["Index"],
+        conn_uniques: Set[UniqueConstraint],
+        conn_indexes: Set[Index],
+        metadata_unique_constraints: Set[UniqueConstraint],
+        metadata_indexes: Set[Index],
     ) -> None:
         pass
 
@@ -564,29 +580,26 @@ class DefaultImpl(metaclass=ImplMeta):
             )
 
     def render_ddl_sql_expr(
-        self, expr: "ClauseElement", is_server_default: bool = False, **kw
+        self, expr: ClauseElement, is_server_default: bool = False, **kw: Any
     ) -> str:
         """Render a SQL expression that is typically a server default,
         index expression, etc.
 
-        .. versionadded:: 1.0.11
-
         """
 
-        compile_kw = dict(
-            compile_kwargs={"literal_binds": True, "include_table": False}
-        )
-        return str(expr.compile(dialect=self.dialect, **compile_kw))
+        compile_kw = {"literal_binds": True, "include_table": False}
 
-    def _compat_autogen_column_reflect(
-        self, inspector: "Inspector"
-    ) -> Callable:
+        return str(
+            expr.compile(dialect=self.dialect, compile_kwargs=compile_kw)
+        )
+
+    def _compat_autogen_column_reflect(self, inspector: Inspector) -> Callable:
         return self.autogen_column_reflect
 
     def correct_for_autogen_foreignkeys(
         self,
-        conn_fks: Set["ForeignKeyConstraint"],
-        metadata_fks: Set["ForeignKeyConstraint"],
+        conn_fks: Set[ForeignKeyConstraint],
+        metadata_fks: Set[ForeignKeyConstraint],
     ) -> None:
         pass
 
@@ -628,19 +641,18 @@ class DefaultImpl(metaclass=ImplMeta):
         self.static_output("COMMIT" + self.command_terminator)
 
     def render_type(
-        self, type_obj: "TypeEngine", autogen_context: "AutogenContext"
-    ) -> Union[str, "Literal[False]"]:
+        self, type_obj: TypeEngine, autogen_context: AutogenContext
+    ) -> Union[str, Literal[False]]:
         return False
 
     def _compare_identity_default(self, metadata_identity, inspector_identity):
-
         # ignored contains the attributes that were not considered
         # because assumed to their default values in the db.
         diff, ignored = _compare_identity_options(
-            sqla_compat._identity_attrs,
             metadata_identity,
             inspector_identity,
             sqla_compat.Identity(),
+            skip={"always"},
         )
 
         meta_always = getattr(metadata_identity, "always", None)
@@ -661,22 +673,172 @@ class DefaultImpl(metaclass=ImplMeta):
             bool(diff) or bool(metadata_identity) != bool(inspector_identity),
         )
 
+    def _compare_index_unique(
+        self, metadata_index: Index, reflected_index: Index
+    ) -> Optional[str]:
+        conn_unique = bool(reflected_index.unique)
+        meta_unique = bool(metadata_index.unique)
+        if conn_unique != meta_unique:
+            return f"unique={conn_unique} to unique={meta_unique}"
+        else:
+            return None
+
+    def _create_metadata_constraint_sig(
+        self, constraint: _autogen._C, **opts: Any
+    ) -> _constraint_sig[_autogen._C]:
+        return _constraint_sig.from_constraint(True, self, constraint, **opts)
+
+    def _create_reflected_constraint_sig(
+        self, constraint: _autogen._C, **opts: Any
+    ) -> _constraint_sig[_autogen._C]:
+        return _constraint_sig.from_constraint(False, self, constraint, **opts)
+
+    def compare_indexes(
+        self,
+        metadata_index: Index,
+        reflected_index: Index,
+    ) -> ComparisonResult:
+        """Compare two indexes by comparing the signature generated by
+        ``create_index_sig``.
+
+        This method returns a ``ComparisonResult``.
+        """
+        msg: List[str] = []
+        unique_msg = self._compare_index_unique(
+            metadata_index, reflected_index
+        )
+        if unique_msg:
+            msg.append(unique_msg)
+        m_sig = self._create_metadata_constraint_sig(metadata_index)
+        r_sig = self._create_reflected_constraint_sig(reflected_index)
+
+        assert _autogen.is_index_sig(m_sig)
+        assert _autogen.is_index_sig(r_sig)
+
+        # The assumption is that the index have no expression
+        for sig in m_sig, r_sig:
+            if sig.has_expressions:
+                log.warning(
+                    "Generating approximate signature for index %s. "
+                    "The dialect "
+                    "implementation should either skip expression indexes "
+                    "or provide a custom implementation.",
+                    sig.const,
+                )
+
+        if m_sig.column_names != r_sig.column_names:
+            msg.append(
+                f"expression {r_sig.column_names} to {m_sig.column_names}"
+            )
+
+        if msg:
+            return ComparisonResult.Different(msg)
+        else:
+            return ComparisonResult.Equal()
+
+    def compare_unique_constraint(
+        self,
+        metadata_constraint: UniqueConstraint,
+        reflected_constraint: UniqueConstraint,
+    ) -> ComparisonResult:
+        """Compare two unique constraints by comparing the two signatures.
+
+        The arguments are two tuples that contain the unique constraint and
+        the signatures generated by ``create_unique_constraint_sig``.
+
+        This method returns a ``ComparisonResult``.
+        """
+        metadata_tup = self._create_metadata_constraint_sig(
+            metadata_constraint
+        )
+        reflected_tup = self._create_reflected_constraint_sig(
+            reflected_constraint
+        )
+
+        meta_sig = metadata_tup.unnamed
+        conn_sig = reflected_tup.unnamed
+        if conn_sig != meta_sig:
+            return ComparisonResult.Different(
+                f"expression {conn_sig} to {meta_sig}"
+            )
+        else:
+            return ComparisonResult.Equal()
+
+    def _skip_functional_indexes(self, metadata_indexes, conn_indexes):
+        conn_indexes_by_name = {c.name: c for c in conn_indexes}
+
+        for idx in list(metadata_indexes):
+            if idx.name in conn_indexes_by_name:
+                continue
+            iex = sqla_compat.is_expression_index(idx)
+            if iex:
+                util.warn(
+                    "autogenerate skipping metadata-specified "
+                    "expression-based index "
+                    f"{idx.name!r}; dialect {self.__dialect__!r} under "
+                    f"SQLAlchemy {sqla_compat.sqlalchemy_version} can't "
+                    "reflect these indexes so they can't be compared"
+                )
+                metadata_indexes.discard(idx)
+
+    def adjust_reflected_dialect_options(
+        self, reflected_object: Dict[str, Any], kind: str
+    ) -> Dict[str, Any]:
+        return reflected_object.get("dialect_options", {})
+
+
+class Params(NamedTuple):
+    token0: str
+    tokens: List[str]
+    args: List[str]
+    kwargs: Dict[str, str]
+
 
 def _compare_identity_options(
-    attributes, metadata_io, inspector_io, default_io
+    metadata_io: Union[schema.Identity, schema.Sequence, None],
+    inspector_io: Union[schema.Identity, schema.Sequence, None],
+    default_io: Union[schema.Identity, schema.Sequence],
+    skip: Set[str],
 ):
     # this can be used for identity or sequence compare.
     # default_io is an instance of IdentityOption with all attributes to the
     # default value.
+    meta_d = sqla_compat._get_identity_options_dict(metadata_io)
+    insp_d = sqla_compat._get_identity_options_dict(inspector_io)
+
     diff = set()
     ignored_attr = set()
-    for attr in attributes:
-        meta_value = getattr(metadata_io, attr, None)
-        default_value = getattr(default_io, attr, None)
-        conn_value = getattr(inspector_io, attr, None)
-        if conn_value != meta_value:
-            if meta_value == default_value:
-                ignored_attr.add(attr)
-            else:
-                diff.add(attr)
+
+    def check_dicts(
+        meta_dict: Mapping[str, Any],
+        insp_dict: Mapping[str, Any],
+        default_dict: Mapping[str, Any],
+        attrs: Iterable[str],
+    ):
+        for attr in set(attrs).difference(skip):
+            meta_value = meta_dict.get(attr)
+            insp_value = insp_dict.get(attr)
+            if insp_value != meta_value:
+                default_value = default_dict.get(attr)
+                if meta_value == default_value:
+                    ignored_attr.add(attr)
+                else:
+                    diff.add(attr)
+
+    check_dicts(
+        meta_d,
+        insp_d,
+        sqla_compat._get_identity_options_dict(default_io),
+        set(meta_d).union(insp_d),
+    )
+    if sqla_compat.identity_has_dialect_kwargs:
+        # use only the dialect kwargs in inspector_io since metadata_io
+        # can have options for many backends
+        check_dicts(
+            getattr(metadata_io, "dialect_kwargs", {}),
+            getattr(inspector_io, "dialect_kwargs", {}),
+            default_io.dialect_kwargs,  # type: ignore[union-attr]
+            getattr(inspector_io, "dialect_kwargs", {}),
+        )
+
     return diff, ignored_attr
