@@ -1,3 +1,6 @@
+# mypy: allow-untyped-defs, allow-incomplete-defs, allow-untyped-calls
+# mypy: no-warn-return-any, allow-any-generics
+
 from __future__ import annotations
 
 from typing import Any
@@ -17,7 +20,7 @@ from sqlalchemy import PrimaryKeyConstraint
 from sqlalchemy import schema as sql_schema
 from sqlalchemy import Table
 from sqlalchemy import types as sqltypes
-from sqlalchemy.events import SchemaEventTarget
+from sqlalchemy.sql.schema import SchemaEventTarget
 from sqlalchemy.util import OrderedDict
 from sqlalchemy.util import topological
 
@@ -33,6 +36,8 @@ from ..util.sqla_compat import _is_type_bound
 from ..util.sqla_compat import _remove_column_from_collection
 from ..util.sqla_compat import _resolve_for_variant
 from ..util.sqla_compat import _select
+from ..util.sqla_compat import constraint_name_defined
+from ..util.sqla_compat import constraint_name_string
 
 if TYPE_CHECKING:
     from typing import Literal
@@ -86,11 +91,11 @@ class BatchOperationsImpl:
         self.batch = []
 
     @property
-    def dialect(self) -> "Dialect":
+    def dialect(self) -> Dialect:
         return self.operations.impl.dialect
 
     @property
-    def impl(self) -> "DefaultImpl":
+    def impl(self) -> DefaultImpl:
         return self.operations.impl
 
     def _should_recreate(self) -> bool:
@@ -174,20 +179,20 @@ class BatchOperationsImpl:
     def drop_column(self, *arg, **kw) -> None:
         self.batch.append(("drop_column", arg, kw))
 
-    def add_constraint(self, const: "Constraint") -> None:
+    def add_constraint(self, const: Constraint) -> None:
         self.batch.append(("add_constraint", (const,), {}))
 
-    def drop_constraint(self, const: "Constraint") -> None:
+    def drop_constraint(self, const: Constraint) -> None:
         self.batch.append(("drop_constraint", (const,), {}))
 
     def rename_table(self, *arg, **kw):
         self.batch.append(("rename_table", arg, kw))
 
-    def create_index(self, idx: "Index") -> None:
-        self.batch.append(("create_index", (idx,), {}))
+    def create_index(self, idx: Index, **kw: Any) -> None:
+        self.batch.append(("create_index", (idx,), kw))
 
-    def drop_index(self, idx: "Index") -> None:
-        self.batch.append(("drop_index", (idx,), {}))
+    def drop_index(self, idx: Index, **kw: Any) -> None:
+        self.batch.append(("drop_index", (idx,), kw))
 
     def create_table_comment(self, table):
         self.batch.append(("create_table_comment", (table,), {}))
@@ -208,8 +213,8 @@ class BatchOperationsImpl:
 class ApplyBatchImpl:
     def __init__(
         self,
-        impl: "DefaultImpl",
-        table: "Table",
+        impl: DefaultImpl,
+        table: Table,
         table_args: tuple,
         table_kwargs: Dict[str, Any],
         reflected: bool,
@@ -236,12 +241,12 @@ class ApplyBatchImpl:
         self._grab_table_elements()
 
     @classmethod
-    def _calc_temp_name(cls, tablename: "quoted_name") -> str:
+    def _calc_temp_name(cls, tablename: Union[quoted_name, str]) -> str:
         return ("_alembic_tmp_%s" % tablename)[0:50]
 
     def _grab_table_elements(self) -> None:
         schema = self.table.schema
-        self.columns: Dict[str, "Column"] = OrderedDict()
+        self.columns: Dict[str, Column[Any]] = OrderedDict()
         for c in self.table.c:
             c_copy = _copy(c, schema=schema)
             c_copy.unique = c_copy.index = False
@@ -250,11 +255,11 @@ class ApplyBatchImpl:
             if isinstance(c.type, SchemaEventTarget):
                 assert c_copy.type is not c.type
             self.columns[c.name] = c_copy
-        self.named_constraints: Dict[str, "Constraint"] = {}
+        self.named_constraints: Dict[str, Constraint] = {}
         self.unnamed_constraints = []
         self.col_named_constraints = {}
-        self.indexes: Dict[str, "Index"] = {}
-        self.new_indexes: Dict[str, "Index"] = {}
+        self.indexes: Dict[str, Index] = {}
+        self.new_indexes: Dict[str, Index] = {}
 
         for const in self.table.constraints:
             if _is_type_bound(const):
@@ -268,7 +273,7 @@ class ApplyBatchImpl:
                 # because
                 # we have no way to determine _is_type_bound() for these.
                 pass
-            elif const.name:
+            elif constraint_name_string(const.name):
                 self.named_constraints[const.name] = const
             else:
                 self.unnamed_constraints.append(const)
@@ -280,7 +285,7 @@ class ApplyBatchImpl:
                         self.col_named_constraints[const.name] = (col, const)
 
         for idx in self.table.indexes:
-            self.indexes[idx.name] = idx
+            self.indexes[idx.name] = idx  # type: ignore[index]
 
         for k in self.table.kwargs:
             self.table_kwargs.setdefault(k, self.table.kwargs[k])
@@ -335,15 +340,12 @@ class ApplyBatchImpl:
         for const in (
             list(self.named_constraints.values()) + self.unnamed_constraints
         ):
-
-            const_columns = set(
-                [c.key for c in _columns_for_constraint(const)]
-            )
+            const_columns = {c.key for c in _columns_for_constraint(const)}
 
             if not const_columns.issubset(self.column_transfers):
                 continue
 
-            const_copy: "Constraint"
+            const_copy: Constraint
             if isinstance(const, ForeignKeyConstraint):
                 if _fk_is_self_referential(const):
                     # for self-referential constraint, refer to the
@@ -368,14 +370,14 @@ class ApplyBatchImpl:
                 self._setup_referent(m, const)
             new_table.append_constraint(const_copy)
 
-    def _gather_indexes_from_both_tables(self) -> List["Index"]:
+    def _gather_indexes_from_both_tables(self) -> List[Index]:
         assert self.new_table is not None
         idx: List[Index] = []
 
         for idx_existing in self.indexes.values():
             # this is a lift-and-move from Table.to_metadata
 
-            if idx_existing._column_flag:  # type: ignore
+            if idx_existing._column_flag:
                 continue
 
             idx_copy = Index(
@@ -402,11 +404,9 @@ class ApplyBatchImpl:
         return idx
 
     def _setup_referent(
-        self, metadata: "MetaData", constraint: "ForeignKeyConstraint"
+        self, metadata: MetaData, constraint: ForeignKeyConstraint
     ) -> None:
-        spec = constraint.elements[
-            0
-        ]._get_colspec()  # type:ignore[attr-defined]
+        spec = constraint.elements[0]._get_colspec()
         parts = spec.split(".")
         tname = parts[-2]
         if len(parts) == 3:
@@ -440,7 +440,7 @@ class ApplyBatchImpl:
                     schema=referent_schema,
                 )
 
-    def _create(self, op_impl: "DefaultImpl") -> None:
+    def _create(self, op_impl: DefaultImpl) -> None:
         self._transfer_elements_to_new_table()
 
         op_impl.prep_table_for_batch(self, self.table)
@@ -484,11 +484,11 @@ class ApplyBatchImpl:
         table_name: str,
         column_name: str,
         nullable: Optional[bool] = None,
-        server_default: Optional[Union["Function", str, bool]] = False,
+        server_default: Optional[Union[Function[Any], str, bool]] = False,
         name: Optional[str] = None,
-        type_: Optional["TypeEngine"] = None,
-        autoincrement: None = None,
-        comment: Union[str, "Literal[False]"] = False,
+        type_: Optional[TypeEngine] = None,
+        autoincrement: Optional[Union[bool, Literal["auto"]]] = None,
+        comment: Union[str, Literal[False]] = False,
         **kw,
     ) -> None:
         existing = self.columns[column_name]
@@ -546,10 +546,8 @@ class ApplyBatchImpl:
                 existing.server_default = None
             else:
                 sql_schema.DefaultClause(
-                    server_default
-                )._set_parent(  # type:ignore[attr-defined]
-                    existing
-                )
+                    server_default  # type: ignore[arg-type]
+                )._set_parent(existing)
         if autoincrement is not None:
             existing.autoincrement = bool(autoincrement)
 
@@ -587,9 +585,9 @@ class ApplyBatchImpl:
                             insert_after = index_cols[idx]
                     else:
                         # insert before a column that is also new
-                        insert_after = dict(
-                            (b, a) for a, b in self.add_col_ordering
-                        )[insert_before]
+                        insert_after = {
+                            b: a for a, b in self.add_col_ordering
+                        }[insert_before]
 
         if insert_before:
             self.add_col_ordering += ((colname, insert_before),)
@@ -607,7 +605,7 @@ class ApplyBatchImpl:
     def add_column(
         self,
         table_name: str,
-        column: "Column",
+        column: Column[Any],
         insert_before: Optional[str] = None,
         insert_after: Optional[str] = None,
         **kw,
@@ -621,7 +619,10 @@ class ApplyBatchImpl:
         self.column_transfers[column.name] = {}
 
     def drop_column(
-        self, table_name: str, column: Union["ColumnClause", "Column"], **kw
+        self,
+        table_name: str,
+        column: Union[ColumnClause[Any], Column[Any]],
+        **kw,
     ) -> None:
         if column.name in self.table.primary_key.columns:
             _remove_column_from_collection(
@@ -663,16 +664,19 @@ class ApplyBatchImpl:
 
         """
 
-    def add_constraint(self, const: "Constraint") -> None:
-        if not const.name:
+    def add_constraint(self, const: Constraint) -> None:
+        if not constraint_name_defined(const.name):
             raise ValueError("Constraint must have a name")
         if isinstance(const, sql_schema.PrimaryKeyConstraint):
             if self.table.primary_key in self.unnamed_constraints:
                 self.unnamed_constraints.remove(self.table.primary_key)
 
-        self.named_constraints[const.name] = const
+        if constraint_name_string(const.name):
+            self.named_constraints[const.name] = const
+        else:
+            self.unnamed_constraints.append(const)
 
-    def drop_constraint(self, const: "Constraint") -> None:
+    def drop_constraint(self, const: Constraint) -> None:
         if not const.name:
             raise ValueError("Constraint must have a name")
         try:
@@ -682,9 +686,11 @@ class ApplyBatchImpl:
                 for col_const in list(self.columns[col.name].constraints):
                     if col_const.name == const.name:
                         self.columns[col.name].constraints.remove(col_const)
-            else:
-                assert const.name
+            elif constraint_name_string(const.name):
                 const = self.named_constraints.pop(const.name)
+            elif const in self.unnamed_constraints:
+                self.unnamed_constraints.remove(const)
+
         except KeyError:
             if _is_type_bound(const):
                 # type-bound constraints are only included in the new
@@ -698,12 +704,12 @@ class ApplyBatchImpl:
                 for col in const.columns:
                     self.columns[col.name].primary_key = False
 
-    def create_index(self, idx: "Index") -> None:
-        self.new_indexes[idx.name] = idx
+    def create_index(self, idx: Index) -> None:
+        self.new_indexes[idx.name] = idx  # type: ignore[index]
 
-    def drop_index(self, idx: "Index") -> None:
+    def drop_index(self, idx: Index) -> None:
         try:
-            del self.indexes[idx.name]
+            del self.indexes[idx.name]  # type: ignore[arg-type]
         except KeyError:
             raise ValueError("No such index: '%s'" % idx.name)
 
